package de.laliluna.example.services;

import de.laliluna.example.model.Phone;
import de.laliluna.example.pages.input.FooValidator;
import de.laliluna.example.security.*;
import de.laliluna.example.util.DoubleSubmitFilter;
import de.laliluna.example.util.PhoneTranslator;
import de.laliluna.example.util.PicoObjectProvider;
import org.apache.tapestry5.*;
import org.apache.tapestry5.ioc.*;
import org.apache.tapestry5.ioc.annotations.Local;
import org.apache.tapestry5.ioc.annotations.Match;
import org.apache.tapestry5.ioc.internal.util.CollectionFactory;
import org.apache.tapestry5.ioc.services.SymbolSource;
import org.apache.tapestry5.services.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.lang.reflect.Method;
import java.util.List;

/**
 * User: hennebrueder
 * Date: 04.08.2009
 */
public class AppModule {

	private final Environment environment;

	public AppModule(Environment environment) {
		this.environment = environment;
	}

	public static void bind(ServiceBinder binder) {
		binder.bind(ServiceLocator.class, ServiceLocatorImpl.class);
		binder.bind(AddressService.class, AddressServiceImpl.class);
		binder.bind(ArticleService.class, ArticleServiceImpl.class);
		binder.bind(CountryService.class, CountryServiceImpl.class);
		binder.bind(SecurityService.class, SecurityServiceImpl.class);
		binder.bind(SampleService.class, SampleServiceImpl.class);
		binder.bind(FormSubmitIdService.class, FormSubmitIdServiceImpl.class);
	}

	@Match("*Service")
	public void adviseNonNull(MethodAdviceReceiver receiver, SecurityService securityService) {
		final Logger logger = LoggerFactory.getLogger(AppModule.class);

		if (receiver.getInterface().equals(SecurityService.class))
			return;

		for (Method m : receiver.getInterface().getMethods()) {
			Protected annotation = m.getAnnotation(Protected.class);
			if (annotation != null && annotation.rights() != null) {
				MethodSecurityAdvice advice = new MethodSecurityAdvice(securityService, annotation.rights());
				receiver.adviseMethod(m, advice);
				logger.debug("Protecting method {} with rights {}", m.getName(), annotation.rights());
			}
		}
	}

	public void contributeApplicationDefaults(
			MappedConfiguration<String, String> configuration) {
		// Contributions to ApplicationDefaults will override any contributions to
		// FactoryDefaults (with the same key). Here we're restricting the supported
		// locales to just "en" (English). As you add localised message catalogs and other assets,
		// you can extend this list of locales (it's a comma separated series of locale names;
		// the first locale name is the default when there's no reasonable match).

		configuration.add(SymbolConstants.SUPPORTED_LOCALES, "en,de");

		// The factory default is true but during the early stages of an application
		// overriding to false is a good idea. In addition, this is often overridden
		// on the command line as -Dtapestry.production-mode=false
		configuration.add(SymbolConstants.PRODUCTION_MODE, "false");
	}

	public SecurityRequestFilter buildSecurityFilter(final SecurityService securityService) {
		return new SecurityRequestFilter(securityService);
	}

	public DoubleSubmitFilter buildDoubleSubmitFilter(final FormSubmitIdService formSubmitIdService){
		return new DoubleSubmitFilter(formSubmitIdService);
	}

	/**
	 * This is a contribution to the RequestHandler service configuration. This is how we extend
	 * Tapestry using the timing filter. A common use for this kind of filter is transaction
	 * management or security. The @Local annotation selects the desired service by type, but only
	 * from the same module.  Without @Local, there would be an error due to the other service(s)
	 * that implement RequestFilter (defined in other modules).
	 */
	public void contributeRequestHandler(OrderedConfiguration<RequestFilter> configuration,
																			 @Local
																			 SecurityRequestFilter filter, @Local DoubleSubmitFilter doubleSubmitFilter) {
		// Each contribution to an ordered configuration has a name, When necessary, you may
		// set constraints to precisely control the invocation order of the contributed filter
		// within the pipeline.

		configuration.add("Security", filter);
		configuration.add("Double submit", doubleSubmitFilter);

	}

	public void contributeValidationMessagesSource(OrderedConfiguration<String> configuration) {
		configuration.add("fooresource", "de/laliluna/example/components/ValidationMessages");
	}

	public void contributeFieldValidatorSource(MappedConfiguration<String, Validator> configuration) {
		configuration.add("foo", new FooValidator());
	}

	public void contributeDefaultDataTypeAnalyzer(MappedConfiguration<Class, String> configuration) {
		configuration.add(Phone.class, "phoneNumber");
	}

	public void contributeBeanBlockSource(Configuration<BeanBlockContribution> configuration) {
		configuration.add(new BeanBlockContribution("phoneNumber", "input/CustomPropertyEditBlocks", "phone", true));
		configuration.add(new BeanBlockContribution("date", "input/CustomPropertyEditBlocks", "date", true));
	}

	public void contributeTranslatorSource(Configuration<Translator> configuration) {
		configuration.add(new PhoneTranslator());
	}

	public static void contributeMasterObjectProvider(OrderedConfiguration<ObjectProvider> configuration){
		configuration.add("PicoObjectProvider", new PicoObjectProvider(), "after:*");
	}

	


	/**
	 * Insert global stylesheets and JavaScript files
	 * @param configuration
	 * @param symbolSource
	 * @param assetSource
	 */
	public void contributeMarkupRenderer(OrderedConfiguration<MarkupRendererFilter> configuration,
																			 final SymbolSource symbolSource, final AssetSource assetSource) {

		MarkupRendererFilter injectDefaultStylesheet = new MarkupRendererFilter() {
			public void renderMarkup(MarkupWriter writer, MarkupRenderer renderer) {
				String globalCss[] = {"de/laliluna/example/components/mystyles.css"};
				RenderSupport renderSupport = environment.peek(RenderSupport.class);

				List<Asset> assets = createAssets(globalCss, symbolSource, assetSource);

				for (Asset stylesheet : assets) {
					renderSupport.addStylesheetLink(stylesheet, null);
				}

				renderer.renderMarkup(writer);
			}
		};

		MarkupRendererFilter injectDefaultJavaScripts = new MarkupRendererFilter() {
			public void renderMarkup(MarkupWriter writer, MarkupRenderer renderer) {
				String globalCss[] = {"de/laliluna/example/components/myscript.js", "de/laliluna/example/pages/input/myvalidators.js"};
				RenderSupport renderSupport = environment.peek(RenderSupport.class);

				List<Asset> assets = createAssets(globalCss, symbolSource, assetSource);
				for (Asset stylesheet : assets) {
					renderSupport.addScriptLink(stylesheet);
				}

				renderer.renderMarkup(writer);
			}
		};


		configuration.add("InjectMyDefaultStyleheet", injectDefaultStylesheet, "after:RenderSupport");
		configuration.add("InjectMyDefaultJavaScripts", injectDefaultJavaScripts, "after:RenderSupport");
	}

	private List<Asset> createAssets(String[] globalCss, SymbolSource symbolSource, AssetSource assetSource) {
		List<Asset> assets = CollectionFactory.newList();

		for (String path : globalCss) {
			String expanded = symbolSource.expandSymbols(path);
			Asset asset = assetSource.getAsset(null, expanded, null);
			assets.add(asset);
		}
		return assets;
	}
}
